@@ -5,7 +5,7 @@ use v5.20;
 use experimental qw/ signatures /;
 package YAML::Tidy;
 
-our $VERSION = '0.001_002'; # TRIAL VERSION
+our $VERSION = '0.002_001'; # TRIAL VERSION
 
 use YAML::Tidy::Node;
 use YAML::Tidy::Config;
@@ -46,6 +46,7 @@ sub tidy($self, $yaml) {
 sub _process($self, $parent, $node) {
     my $type = $node->{type} || '';
     if ($node->{flow}) {
+        $self->_process_flow($parent, $node);
         return;
     }
     my $level = $node->{level};
@@ -65,14 +66,33 @@ sub _process($self, $parent, $node) {
     }
     my $before = substr($line, 0, $col);
 
-    my $pre = $parent ? $parent->pre($node) : undef;
     my $start = $node->start;
-    if ($pre and $trimtrailing) {
-        if (defined $pre->{line} and $pre->{line} <= $start->{line}) {
-            my ($from, $to) = ($pre->{line}, $start->{line});
-            $self->_trim($from, $to);
+    if ($trimtrailing) {
+        my $pre = $parent ? $parent->pre($node) : undef;
+        if ($pre) {
+            if ($pre->{line} <= $start->{line}) {
+                my ($from, $to) = ($pre->{line}, $start->{line});
+                $self->_trim($from, $to);
+            }
         }
-        if ($type eq 'DOC') {
+        if ($level < 1 and $type ne '') {
+            # trim trailing spaces at the end of the node
+            my $last = $node->{children}->[-1];
+            my $from;
+            if ($last) {
+                my $end = $last->endstart;
+                $from = $end->{line} + 1;
+            }
+            else {
+                # empty node
+                $from = $node->{start}->{start}->{line};
+            }
+
+            my $end2 = $node->endstart;
+            my $to = $end2->{line};
+            if ($from <= $to) {
+                $self->_trim($from, $to);
+            }
         }
     }
 
@@ -124,6 +144,7 @@ sub _process($self, $parent, $node) {
         return;
     }
     else {
+        my $ignore_firstlevel = ($self->{partial} and $level == 0);
         my $multiline = $node->multiline;
         if ($parent->{type} eq 'MAP' and ($node->{index} % 2 and not $multiline)) {
             return;
@@ -159,6 +180,7 @@ sub _process($self, $parent, $node) {
         if ($trimtrailing) {
             $self->_trim($startline, $realstart);
         }
+        unless ($ignore_firstlevel) {
         for my $i ($startline .. $realstart) {
             my $line = $lines->[ $i ];
             if ($i == $startline and $col > 0) {
@@ -173,6 +195,7 @@ sub _process($self, $parent, $node) {
             $line =~ s/^ */$new_spaces/;
             $lines->[ $i] = $line;
         }
+        }
         # leave alone explicitly indented block scalars
         return if $explicit_indent;
 
@@ -201,7 +224,7 @@ sub _process($self, $parent, $node) {
             }
             my @slice = @$lines[$startline .. $endline ];
             my ($sp) = $lines->[ $startline ] =~ m/^( *)/;
-            if (length($sp) != $new_indent) {
+            if (not $ignore_firstlevel and length($sp) != $new_indent) {
                 for my $line (@slice) {
                     unless (length $line) {
                         next;
@@ -248,28 +271,44 @@ sub _process($self, $parent, $node) {
             $startline++ if $skipfirst;
             $endline = $node->{end}->{line};
             return if $startline >= @$lines;
+            if ($trimtrailing) {
+                $self->_trim($startline, $endline);
+            }
+            my $line = $lines->[ $startline ];
+            my ($sp) = $line =~ m/^( *)/;
+            if ($ignore_firstlevel) {
+                $new_indent = length $sp;
+                $new_spaces = ' ' x $new_indent;
+            }
             my @slice = @$lines[$startline .. $endline ];
             if ($level == 0 and not $indenttoplevelscalar) {
                 $new_spaces = ' ' x ($new_indent - $indent);
             }
             for my $line (@slice) {
-                if ($line !~ tr/ //c) {
-                    if ($trimtrailing) {
-                        $line = '';
-                    }
-                }
-                else {
+                if ($line =~ tr/ //c) {
                     $line =~ s/^[\t ]*/$new_spaces/;
                 }
-                if ($trimtrailing) {
-                    $line =~ s/[\t ]+$//;
-                }
             }
             @$lines[$startline .. $endline ] = @slice;
         }
     }
 }
 
+sub _process_flow($self, $parent, $node) {
+    return unless $parent;
+    my $flow = $node->{flow};
+    my $indent = $self->cfg->indent;
+    my $lines = $self->{lines};
+    my $col = $node->indent;
+    my $lastcol = $parent->indent;
+    my $realindent = $col - $lastcol;
+    my $startline = $node->line;
+    my $endline = $node->end->{line};
+    if ($flow == 1 and $self->cfg->trimtrailing) {
+        $self->_trim($startline, $endline);
+    }
+}
+
 sub _find_scalar_start($self, $node) {
     my $lines = $self->{lines};
     my $from = $node->line;
@@ -388,6 +427,9 @@ sub _tree($self, $yaml, $lines) {
     $self->{events} = $events;
     my $first = shift @$events;
     my $end = pop @$events;
+    $_->{level} = -1 for ($first, $end);
+    $first->{id} = -1;
+    _pp($first) if DEBUG;
     my @stack;
 
     my $level = -1;
@@ -477,6 +519,8 @@ sub _tree($self, $yaml, $lines) {
         }
         _pp($event) if DEBUG;
     }
+    $end->{id} = $id + 1;
+    _pp($end) if DEBUG;
     $self->{tree} = $docs;
     return $docs;
 }
@@ -546,7 +590,7 @@ __END__
 
 =head1 NAME
 
-YAML::Tidy - Clean YAML files
+YAML::Tidy - Tidy YAML files
 
 =head1 SYNOPSIS
 
@@ -563,7 +607,8 @@ For documentation see L<https://github.com/perlpunk/yamltidy>
 
 =head1 DESCRIPTION
 
-yamltidy can automatically fix indentation in your YAML files.
+yamltidy can automatically tidy formatting in your YAML files, for example
+adjust indentation and remove trailing spaces.
 
 For more information, see L<https://github.com/perlpunk/yamltidy>.
 
