@@ -5,7 +5,7 @@ use v5.20;
 use experimental qw/ signatures /;
 package YAML::Tidy::Node;
 
-our $VERSION = '0.001_002'; # TRIAL VERSION
+our $VERSION = '0.002_001'; # TRIAL VERSION
 
 sub new($class, %args) {
     my $self = {
@@ -50,6 +50,10 @@ sub end($self) {
     return $self->{end}->{end};
 }
 
+sub endstart($self) {
+    return $self->{end}->{start};
+}
+
 sub realendline($self) {
     $self->{end}->{end}->{line} - 1;
 }
@@ -108,6 +112,10 @@ sub end($self) {
     return $self->{end};
 }
 
+sub endstart($self) {
+    return $self->{end};
+}
+
 sub realendline($self) {
     my $end = $self->{end};
     if ($self->{style} == YAML_LITERAL_SCALAR_STYLE
